#include <Python.h>
#include <xmms/xmmsctrl.h>

/* Here is the list of funcs defined by the C library, and any
 * comments/etc.  If they are double-quoted, ie // //, chances
 * are (for whatever reason, usually included) they will not be
 * implemented.
 */

// // void xmms_remote_add_files(session,list) was a define

// void xmms_remote_playlist_ins_url_string(gint session, gchar * string, gint pos\);
static PyObject *xmms_playlist_ins_url_string(PyObject *self,PyObject *args) {
  gchar* url;
  gint pos;

  if(!PyArg_ParseTuple(args,"si",&url, &pos)) return NULL;

  xmms_remote_playlist_ins_url_string(0, url, pos);

  Py_INCREF(Py_None);
  return Py_None;
}

// void xmms_remote_playlist_add(gint session, GList * list);
static PyObject *xmms_playlist_add(PyObject *self,PyObject *args) {
  // Takes a Python list of filenames as the argument.

  // Adding "dir/" or "dir" adds all files under the dir.

  // This even works with Audio CDs, simply by using "/mnt/cdrom" or
  // the like, if XMMS is correctly configured. In fact, it accepts
  // individual tracks in the form "/mnt/cdrom/Track 04.cda" but it's
  // pretty useless because the track list isn't accessible via
  // xmmsctrl.h functions.

  GList* filelist;
  gchar* file;
  gpointer pfile;
  gint i, length;
  PyObject *list;

  //  if(!PyArg_ParseTuple(args,"s",&file)) return NULL;
  if(!PyArg_ParseTuple(args,"O",&list)) return NULL;
  filelist = 0;
  length = PyList_Size(list);
  for (i=0; i<length; i++) {
    file = PyString_AsString(PyList_GetItem(list, i));
    pfile = file;
    filelist = g_list_append(filelist, pfile);
  }

  //  filelist = g_list_append(0, pfile); // the 0 is an empty GList

  xmms_remote_playlist_add(0, filelist);
  g_list_free(filelist);
  Py_INCREF(Py_None);
  return Py_None;
}

// void xmms_remote_playlist_add_url_string(gint session, gchar * string);
static PyObject *xmms_playlist_add_url_string(PyObject *self,PyObject *args) {  gchar* url;

  if(!PyArg_ParseTuple(args,"s",&url)) return NULL;

  xmms_remote_playlist_add_url_string(0, url);

  Py_INCREF(Py_None);
  return Py_None;
}

// void xmms_remote_playlist_delete(gint session, gint pos);
static PyObject *xmms_playlist_delete(PyObject *self,PyObject *args) {
  gint pos;
  if(!PyArg_ParseTuple(args,"i",&pos)) return NULL;
  xmms_remote_playlist_delete(0, pos);
  Py_INCREF(Py_None);
  return Py_None;
}

// void xmms_remote_playlist_clear(gint session);
static PyObject *xmms_playlist_clear(PyObject *self,PyObject *args) {
  xmms_remote_playlist_clear(0);
  Py_INCREF(Py_None);
  return Py_None;
}

// gint xmms_remote_get_playlist_length(gint session);
static PyObject *xmms_get_playlist_length(PyObject *self,PyObject *args) {
  gint length; 
  length = xmms_remote_get_playlist_length(0);
  return Py_BuildValue("i",length);
}

// gint xmms_remote_get_playlist_time(gint session, gint pos);
static PyObject *xmms_get_playlist_time(PyObject *self,PyObject *args) {
  // total length of song
  gint pos;
  gint time, seconds;
  if(!PyArg_ParseTuple(args,"i",&pos)) return NULL;
  time = xmms_remote_get_playlist_time(0, pos);
  seconds = (gint)((gfloat)time / (gfloat)1000);
  return Py_BuildValue("i",seconds);
}

// void xmms_remote_jump_to_time(gint session, gint pos);
static PyObject *xmms_jump_to_time(PyObject *self,PyObject *args) {
  gint seconds, time;
  if(!PyArg_ParseTuple(args,"i",&seconds)) return NULL;
  time = seconds * 1000;
  xmms_remote_jump_to_time(0, time);
  return Py_BuildValue("i",time);
}


// gchar *xmms_remote_get_skin(gint session);
// void xmms_remote_set_skin(gint session, gchar * skinfile);
// void xmms_remote_main_win_toggle(gint session, gboolean show);
// void xmms_remote_pl_win_toggle(gint session, gboolean show);
// void xmms_remote_eq_win_toggle(gint session, gboolean show);
// gboolean xmms_remote_is_main_win(gint session);
// gboolean xmms_remote_is_pl_win(gint session);
// gboolean xmms_remote_is_eq_win(gint session);
// void xmms_remote_show_prefs_box(gint session);
// void xmms_remote_toggle_aot(gint session, gboolean ontop);
// void xmms_remote_eject(gint session);

// gboolean xmms_remote_is_running(gint session);
static PyObject *xmms_is_running(PyObject *self,PyObject *args) {
  gboolean running;
  running = xmms_remote_is_running(0);
  return Py_BuildValue("i",running);
}

// void xmms_remote_toggle_repeat(gint session);
static PyObject *xmms_toggle_repeat(PyObject *self,PyObject *args) {
  xmms_remote_toggle_repeat(0);
  Py_INCREF(Py_None);
  return Py_None;
  }

// void xmms_remote_toggle_shuffle(gint session);
static PyObject *xmms_toggle_shuffle(PyObject *self,PyObject *args) {
  xmms_remote_toggle_shuffle(0);
  Py_INCREF(Py_None);
  return Py_None;
  }

// gboolean xmms_remote_is_repeat(gint session);
static PyObject *xmms_is_repeat(PyObject *self,PyObject *args) {
  gboolean repeat;
  repeat = xmms_remote_is_repeat(0);
  return Py_BuildValue("i",repeat);
}

// gboolean xmms_remote_is_shuffle(gint session);
static PyObject *xmms_is_shuffle(PyObject *self,PyObject *args) {
  gboolean shuffle;
  shuffle = xmms_remote_is_shuffle(0);
  return Py_BuildValue("i",shuffle);
}

// void xmms_remote_get_eq(gint session, gfloat *preamp, gfloat **bands);
// gfloat xmms_remote_get_eq_preamp(gint session);
// gfloat xmms_remote_get_eq_band(gint session, gint band);
// void xmms_remote_set_eq(gint session, gfloat preamp, gfloat *bands);
// void xmms_remote_set_eq_band(gint session, gint band, gfloat value);


// void xmms_remote_set_eq_preamp(gint session, gfloat preamp);
static PyObject *xmms_set_eq_preamp(PyObject *self,PyObject *args) {
  gfloat preamp;

  if(!PyArg_ParseTuple(args,"i",&preamp))
	return NULL;
  xmms_remote_set_eq_preamp(0,preamp);
  Py_INCREF(Py_None);
  return Py_None;
}


/* This is a pretty annoying one, if you are using xmms-pyctl, since
 * there is no xmms_remote_start. Although in theory the ctld could
 * restart it.
 */

// void xmms_remote_quit(gint session);
static PyObject *xmms_quit(PyObject *self,PyObject *args) {
  xmms_remote_quit(0);
  Py_INCREF(Py_None);
  return Py_None;
}

/* The buttons:
 * play, pause, stop, next, prev
 */

static PyObject *xmms_play(PyObject *self,PyObject *args) {
  xmms_remote_play(0);
  
  Py_INCREF(Py_None);
  return Py_None;
}

static PyObject *xmms_pause(PyObject *self,PyObject *args) {
  xmms_remote_pause(0);
  
  Py_INCREF(Py_None);
  return Py_None;
}

static PyObject *xmms_stop(PyObject *self,PyObject *args) {
  xmms_remote_stop(0);
  
  Py_INCREF(Py_None);
  return Py_None;
}
static PyObject *xmms_prev(PyObject *self,PyObject *args) {
  xmms_remote_playlist_prev(0);
  
  Py_INCREF(Py_None);
  return Py_None;
}
static PyObject *xmms_next(PyObject *self,PyObject *args) {
  xmms_remote_playlist_next(0);
  
  Py_INCREF(Py_None);
  return Py_None;
}

/* Some status:
 * is_playing, is_paused, song info, version, get_time
 */

static PyObject *xmms_is_playing(PyObject *self,PyObject *args) {
  gboolean play;
  play=xmms_remote_is_playing(0);
  return Py_BuildValue("i",play);
}

static PyObject *xmms_is_paused(PyObject *self,PyObject *args) {
  gboolean pause;
  pause=xmms_remote_is_paused(0);
  return Py_BuildValue("i",pause);
}

static PyObject *xmms_info(PyObject *self,PyObject *args) {
  //  char *stat;
  gint rate, freq, nch;
  //  PyObject *returnval;
  xmms_remote_get_info(0,&rate,&freq,&nch);
  return Py_BuildValue("iii",rate,freq,nch);
}
	
static PyObject *xmms_get_version(PyObject *self,PyObject *args) {
  gint ver;
  ver=xmms_remote_get_version(0);
  return Py_BuildValue("i",ver);
}

/*  Returns (minutes, seconds) */
static PyObject *xmms_get_time(PyObject *self,PyObject *args) {
  // Change back to seconds. It's more useful for programs...
  gint time,seconds; //,minutes;
  time=xmms_remote_get_output_time(0);
  seconds = (gint)((gfloat)time / (gfloat)1000);
  //  minutes = (gint)((gfloat)seconds / (gfloat)60);
  //  seconds -= minutes * 60;
  return Py_BuildValue("i",seconds);
}

/* Playlist funcs */

// gint xmms_remote_get_playlist_pos(gint session);
static PyObject *xmms_get_playlist_pos(PyObject *self,PyObject *args) {
  gint pos;
  pos=xmms_remote_get_playlist_pos(0);
  //   pos++; // this isn't logical.. see below. /teknohog
  return Py_BuildValue("i",pos);
}

// void xmms_remote_set_playlist_pos(gint session, gint pos);
static PyObject *xmms_set_playlist_pos(PyObject *self, PyObject *args) {
	gint pos;

	if(!PyArg_ParseTuple(args,"i",&pos))
		return NULL;

	// if (pos>1) pos--; 

	// This is a _programming_ interface so we use numbering from
	// 0. It's a matter of the UI if you want to start with 1, I,
	// A, $\alpha$ or whatever.  /teknohog

	xmms_remote_set_playlist_pos(0,pos);
	sleep(1); /* so we get the new value */
	return xmms_get_playlist_pos(self,args);
}

static PyObject *xmms_get_playlist_song(PyObject *self,PyObject *args) {
  gchar *sng;
  sng=xmms_remote_get_playlist_title(0,xmms_remote_get_playlist_pos(0));
  return Py_BuildValue("s",sng);
}

// void xmms_remote_playlist(gint session, gchar ** list, gint num, gboolean enqueue);
// for putting files into the playlist (optionally clearing first)
// static PyObject *xmms_set_playlist(PyObject *self,PyObject *args) {
// 	gchar **list;
// 	gint num;
// 	gboolean enqueue;
// 	int i;
// 	xmms_remote_playlist(0,list,num,enqueue);
// }


// gchar *xmms_remote_get_playlist_file(gint session, gint pos);
// gchar *xmms_remote_get_playlist_title(gint session, gint pos);

static PyObject *xmms_get_playlist_file(PyObject *self,PyObject *args) {
	gint i;
	if(!PyArg_ParseTuple(args,"i",&i))
		return NULL;

	return Py_BuildValue("s",xmms_remote_get_playlist_file(0,i));
}

static PyObject *xmms_get_playlist_title(PyObject *self,PyObject *args) {
	gint i;
	if(!PyArg_ParseTuple(args,"i",&i))
		return NULL;

	return Py_BuildValue("s",xmms_remote_get_playlist_title(0,i));
}


static PyObject *xmms_get_playlist(PyObject *self, PyObject *args) {
	gint i,numrecs;
	gchar *title;
	PyObject *list;

	numrecs=xmms_remote_get_playlist_length(0);
	if(numrecs > 0)
		list = PyList_New(numrecs);
	else
		return Py_BuildValue("i",0);

	for(i=0;i<numrecs;i++) {
		title=xmms_remote_get_playlist_title(0,i);
		if(!strcmp(title,""))
			title=xmms_remote_get_playlist_file(0,i);
		PyList_SetItem(list,i,Py_BuildValue("s",title));
	}
	return list;
}

// void xmms_remote_get_volume(gint session, gint * vl, gint * vr);
static PyObject *xmms_get_volume(PyObject *self, PyObject *args) {
	gint left,right;
	xmms_remote_get_volume(0,&left,&right);
	return Py_BuildValue("ii",left,right);
}
// gint xmms_remote_get_main_volume(gint session);
static PyObject *xmms_get_main_volume(PyObject *self, PyObject *args) {
	gint right;
	right=xmms_remote_get_main_volume(0);
	return Py_BuildValue("i",right);
}
// gint xmms_remote_get_balance(gint session);
static PyObject *xmms_get_balance(PyObject *self, PyObject *args) {
	gint balance;
	balance=xmms_remote_get_balance(0);
	return Py_BuildValue("i",balance);
}
// void xmms_remote_set_volume(gint session, gint vl, gint vr);
static PyObject *xmms_set_volume(PyObject *self, PyObject *args) {
	gint left,right;
	if(!PyArg_ParseTuple(args,"ii",&left,&right))
		return NULL;
	if(left>100) left=100;
	if(left<0) left=0;
	if(right>100) right=100;
	if(right<0) right=0;
	xmms_remote_set_volume(0,left,right);
	return xmms_get_volume(self,args);
}
// void xmms_remote_set_main_volume(gint session, gint v);
static PyObject *xmms_set_main_volume(PyObject *self, PyObject *args) {
	gint right;
	if(!PyArg_ParseTuple(args,"i",&right))
		return NULL;
	if(right>100) right=100;
	if(right<0) right=0;
	xmms_remote_set_main_volume(0,right);
	return xmms_get_main_volume(self,args);
}
// void xmms_remote_set_balance(gint session, gint b);
static PyObject *xmms_set_balance(PyObject *self, PyObject *args) {
	gint balance;
	if(!PyArg_ParseTuple(args,"i",&balance))
		return NULL;
	if(balance>100) balance=100;
	if(balance<0) balance=0;
	xmms_remote_set_balance(0,balance);
	return xmms_get_balance(self,args);
}
	
static PyMethodDef XmmsMethods[] = {
  {"info",  xmms_info, METH_VARARGS},
  {"get_version",  xmms_get_version, METH_VARARGS},
  {"play",  xmms_play, METH_VARARGS},
  {"pause",  xmms_pause, METH_VARARGS},
  {"stop",  xmms_stop, METH_VARARGS},
  {"next",  xmms_next, METH_VARARGS},
  {"prev",  xmms_prev, METH_VARARGS},
  {"quit",  xmms_quit, METH_VARARGS},
  {"is_playing",  xmms_is_playing, METH_VARARGS},
  {"is_paused",  xmms_is_paused, METH_VARARGS},
  {"get_time",  xmms_get_time, METH_VARARGS},
  {"get_playlist_pos",  xmms_get_playlist_pos, METH_VARARGS},
  {"set_playlist_pos",  xmms_set_playlist_pos, METH_VARARGS},
  {"get_playlist_file",  xmms_get_playlist_file, METH_VARARGS},
  {"get_playlist_title",  xmms_get_playlist_title, METH_VARARGS},
  {"get_playlist_song",  xmms_get_playlist_song, METH_VARARGS},
  {"get_playlist",  xmms_get_playlist, METH_VARARGS},
  {"get_volume",  xmms_get_volume, METH_VARARGS},
  {"get_balance",  xmms_get_balance, METH_VARARGS},
  {"get_main_volume",  xmms_get_main_volume, METH_VARARGS},
  {"set_volume",  xmms_set_volume, METH_VARARGS},
  {"set_balance",  xmms_get_volume, METH_VARARGS},
  {"set_main_volume",  xmms_set_main_volume, METH_VARARGS},
  {"set_eq_preamp",  xmms_set_eq_preamp, METH_VARARGS},

  // objects added by Risto A. Paju <teknohog@iki.fi>
  {"toggle_repeat", xmms_toggle_repeat, METH_VARARGS},
  {"toggle_shuffle", xmms_toggle_shuffle, METH_VARARGS},
  {"is_repeat", xmms_is_repeat, METH_VARARGS},
  {"is_shuffle", xmms_is_shuffle, METH_VARARGS},
  {"playlist_add", xmms_playlist_add, METH_VARARGS},
  {"playlist_delete", xmms_playlist_delete, METH_VARARGS},
  {"playlist_clear", xmms_playlist_clear, METH_VARARGS},
  {"get_playlist_length", xmms_get_playlist_length, METH_VARARGS},
  {"playlist_add_url_string", xmms_playlist_add_url_string, METH_VARARGS},
  {"playlist_ins_url_string", xmms_playlist_ins_url_string, METH_VARARGS},
  {"is_running", xmms_is_running, METH_VARARGS},
  {"get_playlist_time", xmms_get_playlist_time, METH_VARARGS},
  {"jump_to_time", xmms_jump_to_time, METH_VARARGS},

  {NULL,      NULL}        /* Sentinel */
};

void initxmms() {
  (void) Py_InitModule("xmms", XmmsMethods);
}


		

